@@ -1,8 +1,10 @@
+/* $Cambridge: exim/exim-src/src/rda.c,v 1.16 2009/11/16 19:50:37 nm4 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2018 */
+/* Copyright (c) University of Cambridge 1995 - 2009 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* This module contains code for extracting addresses from a forwarding list
@@ -110,7 +112,7 @@ if (saved_errno == ENOENT)
   slash = Ustrrchr(big_buffer, '/');
   Ustrcpy(slash+1, ".");
 
-  ALARM(30);
+  alarm(30);
   rc = Ustat(big_buffer, &statbuf);
   if (rc != 0 && errno == EACCES && !sigalrm_seen)
     {
@@ -118,7 +120,7 @@ if (saved_errno == ENOENT)
     rc = Ustat(big_buffer, &statbuf);
     }
   saved_errno = errno;
-  ALARM_CLR(0);
+  alarm(0);
 
   DEBUG(D_route) debug_printf("stat(%s)=%d\n", big_buffer, rc);
   }
@@ -358,7 +360,7 @@ if (rdata->isfile)
   }
 else data = rdata->string;
 
-*filtertype = f.system_filtering ? FILTER_EXIM : rda_is_filter(data);
+*filtertype = system_filtering? FILTER_EXIM : rda_is_filter(data);
 
 /* Filter interpretation is done by a general function that is also called from
 the filter testing option (-bf). There are two versions: one for Exim filtering
@@ -433,24 +435,22 @@ return parse_forward_list(data,
 *         Write string down pipe                 *
 *************************************************/
 
-/* This function is used for transferring a string down a pipe between
+/* This function is used for tranferring a string down a pipe between
 processes. If the pointer is NULL, a length of zero is written.
 
 Arguments:
   fd         the pipe
   s          the string
 
-Returns:     -1 on error, else 0
+Returns:     nothing
 */
 
-static int
-rda_write_string(int fd, const uschar *s)
+static void
+rda_write_string(int fd, uschar *s)
 {
 int len = (s == NULL)? 0 : Ustrlen(s) + 1;
-return (  write(fd, &len, sizeof(int)) != sizeof(int)
-       || (s != NULL  &&  write(fd, s, len) != len)
-       )
-       ? -1 : 0;
+(void)write(fd, &len, sizeof(int));
+if (s != NULL) (void)write(fd, s, len);
 }
 
 
@@ -474,12 +474,11 @@ rda_read_string(int fd, uschar **sp)
 int len;
 
 if (read(fd, &len, sizeof(int)) != sizeof(int)) return FALSE;
-if (len == 0)
-  *sp = NULL;
-else
-  /* We know we have enough memory so disable the error on "len" */
-  /* coverity[tainted_data] */
-  if (read(fd, *sp = store_get(len), len) != len) return FALSE;
+if (len == 0) *sp = NULL; else
+  {
+  *sp = store_get(len);
+  if (read(fd, *sp, len) != len) return FALSE;
+  }
 return TRUE;
 }
 
@@ -492,7 +491,7 @@ return TRUE;
 /* This function is passed a forward list string (unexpanded) or the name of a
 file (unexpanded) whose contents are the forwarding list. The list may in fact
 be a filter program if it starts with "#Exim filter" or "#Sieve filter". Other
-types of filter, with different initial tag strings, may be introduced in due
+types of filter, with different inital tag strings, may be introduced in due
 course.
 
 The job of the function is to process the forwarding list or filter. It is
@@ -566,7 +565,7 @@ DEBUG(D_route) debug_printf("rda_interpret (%s): %s\n",
 data = expand_string(rdata->string);
 if (data == NULL)
   {
-  if (f.expand_string_forcedfail) return FF_NOTDELIVERED;
+  if (expand_string_forcedfail) return FF_NOTDELIVERED;
   *error = string_sprintf("failed to expand \"%s\": %s", rdata->string,
     expand_string_message);
   return FF_ERROR;
@@ -623,13 +622,9 @@ search_tidyup();
 if ((pid = fork()) == 0)
   {
   header_line *waslast = header_last;   /* Save last header */
-  int fd_flags = -1;
 
   fd = pfd[pipe_write];
   (void)close(pfd[pipe_read]);
-
-  if ((fd_flags = fcntl(fd, F_GETFD)) == -1) goto bad;
-  if (fcntl(fd, F_SETFD, fd_flags | FD_CLOEXEC) == -1) goto bad;
   exim_setugid(ugid->uid, ugid->gid, FALSE, rname);
 
   /* Addresses can get rewritten in filters; if we are not root or the exim
@@ -640,7 +635,7 @@ if ((pid = fork()) == 0)
     {
     DEBUG(D_rewrite) debug_printf("turned off address rewrite logging (not "
       "root or exim in this process)\n");
-    BIT_CLEAR(log_selector, log_selector_size, Li_address_rewrite);
+    log_write_selector &= ~L_address_rewrite;
     }
 
   /* Now do the business */
@@ -652,11 +647,9 @@ if ((pid = fork()) == 0)
   /* Pass back whether it was a filter, and the return code and any overall
   error text via the pipe. */
 
-  if (  write(fd, filtertype, sizeof(int)) != sizeof(int)
-     || write(fd, &yield, sizeof(int)) != sizeof(int)
-     || rda_write_string(fd, *error) != 0
-     )
-    goto bad;
+  (void)write(fd, filtertype, sizeof(int));
+  (void)write(fd, &yield, sizeof(int));
+  rda_write_string(fd, *error);
 
   /* Pass back the contents of any syntax error blocks if we have a pointer */
 
@@ -664,50 +657,43 @@ if ((pid = fork()) == 0)
     {
     error_block *ep;
     for (ep = *eblockp; ep != NULL; ep = ep->next)
-      if (  rda_write_string(fd, ep->text1) != 0
-         || rda_write_string(fd, ep->text2) != 0
-	 )
-	goto bad;
-    if (rda_write_string(fd, NULL) != 0)    /* Indicates end of eblocks */
-      goto bad;
+      {
+      rda_write_string(fd, ep->text1);
+      rda_write_string(fd, ep->text2);
+      }
+    rda_write_string(fd, NULL);    /* Indicates end of eblocks */
     }
 
   /* If this is a system filter, we have to pass back the numbers of any
   original header lines that were removed, and then any header lines that were
   added but not subsequently removed. */
 
-  if (f.system_filtering)
+  if (system_filtering)
     {
     int i = 0;
     header_line *h;
     for (h = header_list; h != waslast->next; i++, h = h->next)
-      if (  h->type == htype_old
-         && write(fd, &i, sizeof(i)) != sizeof(i)
-	 )
-	goto bad;
-
+      {
+      if (h->type == htype_old) (void)write(fd, &i, sizeof(i));
+      }
     i = -1;
-    if (write(fd, &i, sizeof(i)) != sizeof(i))
-	goto bad;
+    (void)write(fd, &i, sizeof(i));
 
     while (waslast != header_last)
       {
       waslast = waslast->next;
       if (waslast->type != htype_old)
-	if (  rda_write_string(fd, waslast->text) != 0
-           || write(fd, &(waslast->type), sizeof(waslast->type))
-	      != sizeof(waslast->type)
-	   )
-	  goto bad;
+        {
+        rda_write_string(fd, waslast->text);
+        (void)write(fd, &(waslast->type), sizeof(waslast->type));
+        }
       }
-    if (rda_write_string(fd, NULL) != 0)    /* Indicates end of added headers */
-      goto bad;
+    rda_write_string(fd, NULL);    /* Indicates end of added headers */
     }
 
   /* Write the contents of the $n variables */
 
-  if (write(fd, filter_n, sizeof(filter_n)) != sizeof(filter_n))
-    goto bad;
+  (void)write(fd, filter_n, sizeof(filter_n));
 
   /* If the result was DELIVERED or NOTDELIVERED, we pass back the generated
   addresses, and their associated information, through the pipe. This is
@@ -719,75 +705,56 @@ if ((pid = fork()) == 0)
       yield == FF_FAIL || yield == FF_FREEZE)
     {
     address_item *addr;
-    for (addr = *generated; addr; addr = addr->next)
+    for (addr = *generated; addr != NULL; addr = addr->next)
       {
       int reply_options = 0;
-      int ig_err = addr->prop.ignore_error ? 1 : 0;
 
-      if (  rda_write_string(fd, addr->address) != 0
-         || write(fd, &addr->mode, sizeof(addr->mode)) != sizeof(addr->mode)
-         || write(fd, &addr->flags, sizeof(addr->flags)) != sizeof(addr->flags)
-         || rda_write_string(fd, addr->prop.errors_address) != 0
-         || write(fd, &ig_err, sizeof(ig_err)) != sizeof(ig_err)
-	 )
-	goto bad;
+      rda_write_string(fd, addr->address);
+      (void)write(fd, &(addr->mode), sizeof(addr->mode));
+      (void)write(fd, &(addr->flags), sizeof(addr->flags));
+      rda_write_string(fd, addr->p.errors_address);
 
-      if (addr->pipe_expandn)
+      if (addr->pipe_expandn != NULL)
         {
         uschar **pp;
-        for (pp = addr->pipe_expandn; *pp; pp++)
-          if (rda_write_string(fd, *pp) != 0)
-	    goto bad;
+        for (pp = addr->pipe_expandn; *pp != NULL; pp++)
+          rda_write_string(fd, *pp);
         }
-      if (rda_write_string(fd, NULL) != 0)
-        goto bad;
+      rda_write_string(fd, NULL);
 
-      if (!addr->reply)
-	{
-        if (write(fd, &reply_options, sizeof(int)) != sizeof(int))    /* 0 means no reply */
-	  goto bad;
-	}
+      if (addr->reply == NULL)
+        (void)write(fd, &reply_options, sizeof(int));    /* 0 means no reply */
       else
         {
         reply_options |= REPLY_EXISTS;
         if (addr->reply->file_expand) reply_options |= REPLY_EXPAND;
         if (addr->reply->return_message) reply_options |= REPLY_RETURN;
-        if (  write(fd, &reply_options, sizeof(int)) != sizeof(int)
-           || write(fd, &(addr->reply->expand_forbid), sizeof(int))
-	      != sizeof(int)
-           || write(fd, &(addr->reply->once_repeat), sizeof(time_t))
-	      != sizeof(time_t)
-           || rda_write_string(fd, addr->reply->to) != 0
-           || rda_write_string(fd, addr->reply->cc) != 0
-           || rda_write_string(fd, addr->reply->bcc) != 0
-           || rda_write_string(fd, addr->reply->from) != 0
-           || rda_write_string(fd, addr->reply->reply_to) != 0
-           || rda_write_string(fd, addr->reply->subject) != 0
-           || rda_write_string(fd, addr->reply->headers) != 0
-           || rda_write_string(fd, addr->reply->text) != 0
-           || rda_write_string(fd, addr->reply->file) != 0
-           || rda_write_string(fd, addr->reply->logfile) != 0
-           || rda_write_string(fd, addr->reply->oncelog) != 0
-	   )
-	  goto bad;
+        (void)write(fd, &reply_options, sizeof(int));
+        (void)write(fd, &(addr->reply->expand_forbid), sizeof(int));
+        (void)write(fd, &(addr->reply->once_repeat), sizeof(time_t));
+        rda_write_string(fd, addr->reply->to);
+        rda_write_string(fd, addr->reply->cc);
+        rda_write_string(fd, addr->reply->bcc);
+        rda_write_string(fd, addr->reply->from);
+        rda_write_string(fd, addr->reply->reply_to);
+        rda_write_string(fd, addr->reply->subject);
+        rda_write_string(fd, addr->reply->headers);
+        rda_write_string(fd, addr->reply->text);
+        rda_write_string(fd, addr->reply->file);
+        rda_write_string(fd, addr->reply->logfile);
+        rda_write_string(fd, addr->reply->oncelog);
         }
       }
 
-    if (rda_write_string(fd, NULL) != 0)   /* Marks end of addresses */
-      goto bad;
+    rda_write_string(fd, NULL);   /* Marks end of addresses */
     }
 
   /* OK, this process is now done. Free any cached resources. Must use _exit()
   and not exit() !! */
 
-out:
   (void)close(fd);
   search_tidyup();
   _exit(0);
-
-bad:
-  DEBUG(D_rewrite) debug_printf("rda_interpret: failed write to pipe\n");
-  goto out;
   }
 
 /* Back in the main process: panic if the fork did not succeed. */
@@ -810,28 +777,29 @@ if (read(fd, filtertype, sizeof(int)) != sizeof(int) ||
 
 /* Read the contents of any syntax error blocks if we have a pointer */
 
-if (eblockp)
+if (eblockp != NULL)
   {
+  uschar *s;
   error_block *e;
-  error_block **p;
-  for (p = eblockp; ; p = &e->next)
+  error_block **p = eblockp;
+  for (;;)
     {
-    uschar *s;
     if (!rda_read_string(fd, &s)) goto DISASTER;
-    if (!s) break;
+    if (s == NULL) break;
     e = store_get(sizeof(error_block));
     e->next = NULL;
     e->text1 = s;
     if (!rda_read_string(fd, &s)) goto DISASTER;
     e->text2 = s;
     *p = e;
+    p = &(e->next);
     }
   }
 
 /* If this is a system filter, read the identify of any original header lines
 that were removed, and then read data for any new ones that were added. */
 
-if (f.system_filtering)
+if (system_filtering)
   {
   int hn = 0;
   header_line *h = header_list;
@@ -844,7 +812,8 @@ if (f.system_filtering)
     while (hn < n)
       {
       hn++;
-      if (!(h = h->next)) goto DISASTER_NO_HEADER;
+      h = h->next;
+      if (h == NULL) goto DISASTER_NO_HEADER;
       }
     h->type = htype_old;
     }
@@ -854,7 +823,7 @@ if (f.system_filtering)
     uschar *s;
     int type;
     if (!rda_read_string(fd, &s)) goto DISASTER;
-    if (!s) break;
+    if (s == NULL) break;
     if (read(fd, &type, sizeof(type)) != sizeof(type)) goto DISASTER;
     header_add(type, "%s", s);
     }
@@ -893,13 +862,9 @@ if (yield == FF_DELIVERED || yield == FF_NOTDELIVERED ||
 
     /* Next comes the mode and the flags fields */
 
-    if (  read(fd, &addr->mode, sizeof(addr->mode)) != sizeof(addr->mode)
-       || read(fd, &addr->flags, sizeof(addr->flags)) != sizeof(addr->flags)
-       || !rda_read_string(fd, &addr->prop.errors_address)
-       || read(fd, &i, sizeof(i)) != sizeof(i)
-       )
-      goto DISASTER;
-    addr->prop.ignore_error = (i != 0);
+    if (read(fd, &(addr->mode), sizeof(addr->mode)) != sizeof(addr->mode) ||
+        read(fd, &(addr->flags), sizeof(addr->flags)) != sizeof(addr->flags) ||
+        !rda_read_string(fd, &(addr->p.errors_address))) goto DISASTER;
 
     /* Next comes a possible setting for $thisaddress and any numerical
     variables for pipe expansion, terminated by a NULL string. The maximum
@@ -918,7 +883,7 @@ if (yield == FF_DELIVERED || yield == FF_NOTDELIVERED ||
 
     if (i > 0)
       {
-      addr->pipe_expandn = store_get((i+1) * sizeof(uschar *));
+      addr->pipe_expandn = store_get((i+1) * sizeof(uschar **));
       addr->pipe_expandn[i] = NULL;
       while (--i >= 0) addr->pipe_expandn[i] = expandn[i];
       }
