@@ -1,4 +1,4 @@
-/* $Cambridge: exim/exim-src/src/spam.c,v 1.11 2005/08/01 14:41:25 ph10 Exp $ */
+/* $Cambridge: exim/exim-src/src/spam.c,v 1.17 2008/07/18 17:55:42 fanf2 Exp $ */
 
 /*************************************************
 *     Exim - an Internet mail transport agent    *
@@ -42,10 +42,14 @@ int spam(uschar **listptr) {
   struct sockaddr_un server;
 #ifndef NO_POLL_H
   struct pollfd pollfd;
+#else                               /* Patch posted by Erik ? for OS X */
+  struct timeval select_tv;         /* and applied by PH */
+  fd_set select_fd;
 #endif
+  uschar *spamd_address_work;
 
   /* stop compiler warning */
-  result = result;
+  result = 0;
 
   /* find the username from the option list */
   if ((user_name = string_nextinlist(&list, &sep,
@@ -86,14 +90,26 @@ int spam(uschar **listptr) {
   };
 
   start = time(NULL);
+
+  if (*spamd_address == '$') {
+    spamd_address_work = expand_string(spamd_address);
+    if (spamd_address_work == NULL) {
+      log_write(0, LOG_MAIN|LOG_PANIC,
+        "spamassassin acl condition: spamd_address starts with $, but expansion failed: %s", expand_string_message);
+      return DEFER;
+    }
+  }
+  else
+    spamd_address_work = spamd_address;
+
   /* socket does not start with '/' -> network socket */
-  if (*spamd_address != '/') {
+  if (*spamd_address_work != '/') {
     time_t now = time(NULL);
     int num_servers = 0;
     int current_server = 0;
     int start_server = 0;
     uschar *address = NULL;
-    uschar *spamd_address_list_ptr = spamd_address;
+    uschar *spamd_address_list_ptr = spamd_address_work;
     uschar address_buffer[256];
     spamd_address_container * spamd_address_vector[32];
 
@@ -181,12 +197,12 @@ int spam(uschar **listptr) {
     }
 
     server.sun_family = AF_UNIX;
-    Ustrcpy(server.sun_path, spamd_address);
+    Ustrcpy(server.sun_path, spamd_address_work);
 
     if (connect(spamd_sock, (struct sockaddr *) &server, sizeof(struct sockaddr_un)) < 0) {
       log_write(0, LOG_MAIN|LOG_PANIC,
                 "malware acl condition: spamd: unable to connect to UNIX socket %s (%s)",
-                spamd_address, strerror(errno) );
+                spamd_address_work, strerror(errno) );
       (void)fclose(mbox_file);
       (void)close(spamd_sock);
       return DEFER;
@@ -218,7 +234,8 @@ int spam(uschar **listptr) {
    * and we poll the desciptor to make sure that we can write without
    * blocking.  Short writes are gracefully handled and if the whole
    * trasaction takes too long it is aborted.
-   * Note: poll() is not supported in OSX 10.2.
+   * Note: poll() is not supported in OSX 10.2 and is reported to be
+   *       broken in more recent versions (up to 10.4).
    */
 #ifndef NO_POLL_H
   pollfd.fd = spamd_sock;
@@ -232,8 +249,19 @@ int spam(uschar **listptr) {
 again:
 #ifndef NO_POLL_H
       result = poll(&pollfd, 1, 1000);
+
+/* Patch posted by Erik ? for OS X and applied by PH */
+#else
+      select_tv.tv_sec = 1;
+      select_tv.tv_usec = 0;
+      FD_ZERO(&select_fd);
+      FD_SET(spamd_sock, &select_fd);
+      result = select(spamd_sock+1, NULL, &select_fd, NULL, &select_tv);
+#endif
+/* End Erik's patch */
+
       if (result == -1 && errno == EINTR)
-        continue;
+        goto again;
       else if (result < 1) {
         if (result == -1)
           log_write(0, LOG_MAIN|LOG_PANIC,
@@ -248,7 +276,7 @@ again:
         (void)fclose(mbox_file);
         return DEFER;
       }
-#endif
+
       wrote = send(spamd_sock,spamd_buffer + offset,read - offset,0);
       if (wrote == -1)
       {
@@ -301,11 +329,11 @@ again:
   (void)close(spamd_sock);
 
   /* dig in the spamd output and put the report in a multiline header, if requested */
-  if( sscanf(CS spamd_buffer,"SPAMD/%s 0 EX_OK\r\nContent-length: %*u\r\n\r\n%lf/%lf\r\n%n",
+  if( sscanf(CS spamd_buffer,"SPAMD/%7s 0 EX_OK\r\nContent-length: %*u\r\n\r\n%lf/%lf\r\n%n",
              spamd_version,&spamd_score,&spamd_threshold,&spamd_report_offset) != 3 ) {
 
     /* try to fall back to pre-2.50 spamd output */
-    if( sscanf(CS spamd_buffer,"SPAMD/%s 0 EX_OK\r\nSpam: %*s ; %lf / %lf\r\n\r\n%n",
+    if( sscanf(CS spamd_buffer,"SPAMD/%7s 0 EX_OK\r\nSpam: %*s ; %lf / %lf\r\n\r\n%n",
                spamd_version,&spamd_score,&spamd_threshold,&spamd_report_offset) != 3 ) {
       log_write(0, LOG_MAIN|LOG_PANIC,
          "spam acl condition: cannot parse spamd output");
@@ -326,13 +354,10 @@ again:
     *q = *p;
     q++;
     if (*p == '\n') {
-      *q = '\t';
+      /* add an extra space after the newline to ensure
+      that it is treated as a header continuation line */
+      *q = ' ';
       q++;
-      /* eat whitespace */
-      while( (*p <= ' ') && (*p != '\0') ) {
-        p++;
-      };
-      p--;
     };
     p++;
   };
@@ -380,9 +405,11 @@ again:
     spam_rc = FAIL;
   };
 
-  /* remember user name and "been here" for it */
-  Ustrcpy(prev_user_name, user_name);
-  spam_ok = 1;
+  /* remember user name and "been here" for it unless spamd_socket was expanded */
+  if (spamd_address_work == spamd_address) {
+    Ustrcpy(prev_user_name, user_name);
+    spam_ok = 1;
+  }
 
   if (override) {
     /* always return OK, no matter what the score */
